// Emacs style mode select -*- C++ -*-
//---------------------------------------------------------------------------
//
// Copyright(C) 2000 James Haley
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//--------------------------------------------------------------------------
//
// Code for handling Hexen-style skies -- parallax, scrolling,
// taller textures. Nice stuff. These drawing routines take over only
// in sectors that use the new F_SKY2 flat -- to avoid serious 
// complications, F_SKY1 sectors cannot make use of these features.
// Also, sky transfer linedefs will not work on F_SKY2 sectors. There's
// very little reason why the two would ever be mixed side-by-side,
// and definitely no reason for needing both features in the same
// sector.
//
// The sky column offsets for scrolling are updated by P_AnimateSurfaces
// in p_anim.c (same function that drives lightning).
//
// By Quasar
//
//---------------------------------------------------------------------------

#include "doomstat.h"
#include "doomtype.h"
#include "r_data.h"
#include "r_defs.h"
#include "r_draw.h"
#include "r_main.h"
#include "r_sky.h"

extern void R_DrawNewSkyColumn(void);

void do_draw_newsky(visplane_t *pl)
{
   int x, offset, skyTexture, offset2, skyTexture2;
   angle_t an;
	
   an = viewangle;

   if(DoubleSky) // render two layers
   {
      // get scrolling offsets and textures
      offset = Sky1ColumnOffset>>16;
      skyTexture = texturetranslation[skytexture];
      offset2 = Sky2ColumnOffset>>16;
      skyTexture2 = texturetranslation[sky2texture];

      dc_texturemid = skytexturemid;    // Default y-offset

      if(comp[comp_skymap] || !(dc_colormap = fixedcolormap))
        dc_colormap = fullcolormap;

      // first draw sky 2 with R_DrawColumn (unmasked)

      dc_texheight = (textureheight[skyTexture2])>>FRACBITS;
      dc_iscale = pspriteiscale>>stretchsky;

      for(x = pl->minx; (dc_x = x) <= pl->maxx; x++)
        if ((dc_yl = pl->top[x]) <= (dc_yh = pl->bottom[x]))
        {
           dc_source =
             R_GetColumn(skyTexture2,
               ((an + xtoviewangle[x] + offset2)) >> (ANGLETOSKYSHIFT));
           colfunc();
        }

      // now draw sky 1 with R_DrawNewSkyColumn (masked)
      dc_texheight = (textureheight[skyTexture])>>FRACBITS;

      for(x = pl->minx; (dc_x = x) <= pl->maxx; x++)
        if ((dc_yl = pl->top[x]) <= (dc_yh = pl->bottom[x]))
        {
           dc_source =
             R_GetColumn(skyTexture,
               ((an + xtoviewangle[x] + offset)) >> (ANGLETOSKYSHIFT));
           R_DrawNewSkyColumn();
        }
   }
   else // one layer only
   {
      offset = Sky1ColumnOffset>>16;
      skyTexture = texturetranslation[skytexture];

      dc_texturemid = skytexturemid;    // Default y-offset

      if(comp[comp_skymap] || !(dc_colormap = fixedcolormap))
        dc_colormap = fullcolormap;

      dc_texheight = (textureheight[skyTexture])>>FRACBITS;
      dc_iscale = pspriteiscale>>stretchsky;

      for(x = pl->minx; (dc_x = x) <= pl->maxx; x++)
        if ((dc_yl = pl->top[x]) <= (dc_yh = pl->bottom[x]))
        {
           dc_source =
             R_GetColumn(skyTexture,
               ((an + xtoviewangle[x] + offset)) >> (ANGLETOSKYSHIFT));
           colfunc();
        }
   }
}

/*
void do_draw_newsky(visplane_t *pl)
{
   int x, angle;
   byte *source, *source2, *dest;
   int count, offset, skyTexture, offset2, skyTexture2;

   if(DoubleSky) // parallax skies
   {
      offset = Sky1ColumnOffset>>16;
      skyTexture = texturetranslation[skytexture];
      offset2 = Sky2ColumnOffset>>16;
      skyTexture2 = texturetranslation[sky2texture];
      for(x = pl->minx; x <= pl->maxx; x++)
      {
	 dc_yl = pl->top[x];
	 dc_yh = pl->bottom[x];
	 if(dc_yl <= dc_yh)
	 {
	    count = dc_yh - dc_yl;
	    if(count < 0)
	    {
	       return;
	    }
	    angle = (viewangle + xtoviewangle[x])>>ANGLETOSKYSHIFT;
	    source = R_GetColumn(skyTexture, angle + offset) + 
	              200 + (dc_yl - centery);
	    source2 = R_GetColumn(skyTexture2, angle + offset2) + 
	               200 + (dc_yl - centery);
	    dest = ylookup[dc_yl] + columnofs[x];
	    do
	    {
	       if(*source)
	       {
		  *dest = *source++;
		  source2++;
	       }
	       else
	       {
		  *dest = *source2++;
		  source++;
	       }
	       dest += SCREENWIDTH<<hires;
	    } while(count--);
	 }
      }
   }
   else
   { // Render single layer
      
//      if(pl->special == 200)
//      { // Use sky 2
//         offset = Sky2ColumnOffset>>16;
//         skyTexture = texturetranslation[sky2texture];
//      }
//      else
//      { // Use sky 1

      offset = Sky1ColumnOffset>>16;
      skyTexture = texturetranslation[skytexture];
 
      for(x = pl->minx; x <= pl->maxx; x++)
      {
	 dc_yl = pl->top[x];
	 dc_yh = pl->bottom[x];
         if(dc_yl <= dc_yh)
	 {
	    count = dc_yh - dc_yl;
	    if(count < 0)
	    {
	       return;
	    }
	    angle = (viewangle + xtoviewangle[x])>>ANGLETOSKYSHIFT;
	    source = R_GetColumn(skyTexture, angle+offset) + 
		       200 + (dc_yl - centery);
	    dest = ylookup[dc_yl] + columnofs[x];
	    do
	    {
	       *dest = *source++;
	       dest += SCREENWIDTH<<hires;
	    } while(count--);
	 }
      }
   }
}
*/
